function [w, mz, llh]=probitem(in, trg, prec, maxdiff, maxcnt, lambda)
% function [w, mz, llh]=probitem(in, trg, prec, maxdiff, maxcnt, lambda)
% implements EM algorithm to maximise the 
% binomial likelihood w.r.t. parameters of 
% a generalized linear probit model.
%
% Params :
% in, trg :        inputs and binary class labels as targets
% prec:            precision of the normal used in the truncated Gaussian in
%                  latent space (i.e. precision of p(z|x'w))
% maxdiff, maxcnt: control optiomisation.
% lambda:          (optional) precision of parameter prior.
%
% Output: 
% w :              the optimal parameter value of the rgeression model
% mz :             map value of the latent variable (i.e. in*w)
% llh :             log likelihood at maximum
%
% (C) P. Sykacek 2004

%# scalar cnt int_maxcnt diff int_maxdiff

if nargin == 5 % the we have a non penalized ML solution
    lambda=0;
end
if max(trg)==2
  trg=trg-1;
end
[nsmpl, nin]=size(in);
w=zeros(nin,1);
std_z=1/sqrt(prec);
oldLh=-inf;
diff=inf;
cnt=0;
int_maxcnt=maxcnt;
int_maxdiff=maxdiff;
%allhs=[];
while (cnt < int_maxcnt) & (diff > int_maxdiff)
  [mz]=em_Qzn(w, in);
  [Ez]=em_EQz(mz, std_z, in, trg);
  [w, P, C]=em_Qw(in, Ez, lambda, prec); % P and C are ignored here. (precision and covarioance)
  Lh=mz-in*w;
  Lh=-Lh'*Lh;
  diff=abs(oldLh-Lh);
  %disp(diff);
  oldLh=Lh;
  %disp(w')
  cnt=cnt+1;
  %allhs(cnt)=Lh;
end
allPs=cdf_Gauss(zeros(size(mz)), mz, ones(size(mz))*prec);
id_one=find(trg==1);
allPs(id_one)=1-allPs(id_one);
llh=sum(log(allPs));
%mz=-mz;
